package de.juplo.jpa.converters;

import de.juplo.jpa.converters.TimeConversions.FixedZoneIdStrategy;
import de.juplo.jpa.converters.TimeConversions.FixedZoneOffsetStrategy;
import de.juplo.yourshouter.api.model.Date;
import de.juplo.yourshouter.api.persistence.jpa.PrimaryKey;
import java.net.URI;
import java.time.LocalDate;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import org.junit.Test;
import static org.junit.Assert.assertEquals;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 *
 * @author kai
 */
public class JPAConverterTest
{
  final static Logger log = LoggerFactory.getLogger(JPAConverterTest.class);


  EntityManager em;

  ZonedDateTime zdt = ZonedDateTime.now();
  OffsetDateTime odt = OffsetDateTime.now();
  LocalDate ld = LocalDate.now();

  ZoneId defaultZone = ZoneId.systemDefault();
  ZoneId otherZone = defaultZone.getId().equals("America/New_York") ? ZoneId.of("Europe/Berlin") : ZoneId.of("America/New_York");

  ZoneOffset defaultOffset = defaultZone.getRules().getOffset(odt.toLocalDateTime());
  ZoneOffset otherOffset = ZoneOffset.ofTotalSeconds(defaultOffset.getTotalSeconds() + 3 * 60 * 60);


  @Test
  public void testOffsetDateTimeAnnotated()
  {
    em = Persistence.createEntityManagerFactory("annotated").createEntityManager();
    testOffsetDateTime();
  }

  @Test
  public void testOffsetDateTimeXml()
  {
    em = Persistence.createEntityManagerFactory("xml").createEntityManager();
    testOffsetDateTime();
  }

  public void testOffsetDateTime()
  {
    EierlegendeWollmilchSau ews;

    TimeConversions.setZoneOffsetStrategy(new FixedZoneOffsetStrategy(defaultOffset));
    ews = new EierlegendeWollmilchSau();
    ews.odt = odt;
    persist(ews);
    ews = retrieve(EierlegendeWollmilchSau.class, ews.id);
    assertEquals(odt, ews.odt);

    TimeConversions.setZoneOffsetStrategy(new FixedZoneOffsetStrategy(otherOffset));
    ews = new EierlegendeWollmilchSau();
    ews.odt = odt;
    persist(ews);
    ews = retrieve(EierlegendeWollmilchSau.class, ews.id);
    assertEquals(odt.toInstant(), ews.odt.toInstant());
  }


  @Test
  public void testZonedDateTimeAnnotated()
  {
    em = Persistence.createEntityManagerFactory("annotated").createEntityManager();
    testZonedDateTime();
  }

  @Test
  public void testZonedDateTimeXml()
  {
    em = Persistence.createEntityManagerFactory("xml").createEntityManager();
    testZonedDateTime();
  }

  public void testZonedDateTime()
  {
    EierlegendeWollmilchSau ews;

    TimeConversions.setZoneIdStrategy(new FixedZoneIdStrategy(defaultZone));
    ews = new EierlegendeWollmilchSau();
    ews.zdt = zdt;
    persist(ews);
    ews = retrieve(EierlegendeWollmilchSau.class, ews.id);
    assertEquals(zdt, ews.zdt);

    TimeConversions.setZoneIdStrategy(new FixedZoneIdStrategy(otherZone));
    ews = new EierlegendeWollmilchSau();
    ews.zdt = zdt;
    persist(ews);
    ews = retrieve(EierlegendeWollmilchSau.class, ews.id);
    assertEquals(zdt.toInstant(), ews.zdt.toInstant());
  }


  @Test
  public void testDataModel()
  {
    em = Persistence.createEntityManagerFactory("data-model").createEntityManager();
    Date date;

    TimeConversions.setZoneIdStrategy(new FixedZoneIdStrategy(defaultZone));
    date = new Date();
    date.setSource(URI.create("http://foo.bar"));
    date.setId("TEST");
    date.setDate(ld);
    date.setStart(zdt);
    date.setEnd(zdt);
    persist(date);
    date = retrieve(Date.class, new PrimaryKey(date));
    assertEquals(ld, date.getDate());
    assertEquals(zdt, date.getStart());
    assertEquals(zdt, date.getEnd());
  }


  private void persist(Object entity)
  {
    EntityTransaction tx = em.getTransaction();
    tx.begin();
    em.persist(entity);
    log.info("persisting {}", entity);
    tx.commit();
    em.clear();
  }

  private <T> T retrieve(Class<T> entityClass, Object id)
  {
    EntityTransaction tx = em.getTransaction();
    tx.begin();
    try
    {
      log.info("retrieving {} of class {}", id, entityClass);
      return em.find(entityClass, id);
    }
    finally
    {
      tx.rollback();
    }
  }
}
